# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.retail_v2.types import common
from google.cloud.retail_v2.types import product as gcr_product

__protobuf__ = proto.module(
    package="google.cloud.retail.v2",
    manifest={
        "CreateProductRequest",
        "GetProductRequest",
        "UpdateProductRequest",
        "DeleteProductRequest",
        "ListProductsRequest",
        "ListProductsResponse",
        "SetInventoryRequest",
        "SetInventoryMetadata",
        "SetInventoryResponse",
        "AddFulfillmentPlacesRequest",
        "AddFulfillmentPlacesMetadata",
        "AddFulfillmentPlacesResponse",
        "AddLocalInventoriesRequest",
        "AddLocalInventoriesMetadata",
        "AddLocalInventoriesResponse",
        "RemoveLocalInventoriesRequest",
        "RemoveLocalInventoriesMetadata",
        "RemoveLocalInventoriesResponse",
        "RemoveFulfillmentPlacesRequest",
        "RemoveFulfillmentPlacesMetadata",
        "RemoveFulfillmentPlacesResponse",
    },
)


class CreateProductRequest(proto.Message):
    r"""Request message for
    [ProductService.CreateProduct][google.cloud.retail.v2.ProductService.CreateProduct]
    method.

    Attributes:
        parent (str):
            Required. The parent catalog resource name, such as
            ``projects/*/locations/global/catalogs/default_catalog/branches/default_branch``.
        product (google.cloud.retail_v2.types.Product):
            Required. The [Product][google.cloud.retail.v2.Product] to
            create.
        product_id (str):
            Required. The ID to use for the
            [Product][google.cloud.retail.v2.Product], which will become
            the final component of the
            [Product.name][google.cloud.retail.v2.Product.name].

            If the caller does not have permission to create the
            [Product][google.cloud.retail.v2.Product], regardless of
            whether or not it exists, a PERMISSION_DENIED error is
            returned.

            This field must be unique among all
            [Product][google.cloud.retail.v2.Product]s with the same
            [parent][google.cloud.retail.v2.CreateProductRequest.parent].
            Otherwise, an ALREADY_EXISTS error is returned.

            This field must be a UTF-8 encoded string with a length
            limit of 128 characters. Otherwise, an INVALID_ARGUMENT
            error is returned.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    product: gcr_product.Product = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gcr_product.Product,
    )
    product_id: str = proto.Field(
        proto.STRING,
        number=3,
    )


class GetProductRequest(proto.Message):
    r"""Request message for
    [ProductService.GetProduct][google.cloud.retail.v2.ProductService.GetProduct]
    method.

    Attributes:
        name (str):
            Required. Full resource name of
            [Product][google.cloud.retail.v2.Product], such as
            ``projects/*/locations/global/catalogs/default_catalog/branches/default_branch/products/some_product_id``.

            If the caller does not have permission to access the
            [Product][google.cloud.retail.v2.Product], regardless of
            whether or not it exists, a PERMISSION_DENIED error is
            returned.

            If the requested [Product][google.cloud.retail.v2.Product]
            does not exist, a NOT_FOUND error is returned.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class UpdateProductRequest(proto.Message):
    r"""Request message for
    [ProductService.UpdateProduct][google.cloud.retail.v2.ProductService.UpdateProduct]
    method.

    Attributes:
        product (google.cloud.retail_v2.types.Product):
            Required. The product to update/create.

            If the caller does not have permission to update the
            [Product][google.cloud.retail.v2.Product], regardless of
            whether or not it exists, a PERMISSION_DENIED error is
            returned.

            If the [Product][google.cloud.retail.v2.Product] to update
            does not exist and
            [allow_missing][google.cloud.retail.v2.UpdateProductRequest.allow_missing]
            is not set, a NOT_FOUND error is returned.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Indicates which fields in the provided
            [Product][google.cloud.retail.v2.Product] to update. The
            immutable and output only fields are NOT supported. If not
            set, all supported fields (the fields that are neither
            immutable nor output only) are updated.

            If an unsupported or unknown field is provided, an
            INVALID_ARGUMENT error is returned.

            The attribute key can be updated by setting the mask path as
            "attributes.${key_name}". If a key name is present in the
            mask but not in the patching product from the request, this
            key will be deleted after the update.
        allow_missing (bool):
            If set to true, and the
            [Product][google.cloud.retail.v2.Product] is not found, a
            new [Product][google.cloud.retail.v2.Product] will be
            created. In this situation, ``update_mask`` is ignored.
    """

    product: gcr_product.Product = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gcr_product.Product,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )
    allow_missing: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class DeleteProductRequest(proto.Message):
    r"""Request message for
    [ProductService.DeleteProduct][google.cloud.retail.v2.ProductService.DeleteProduct]
    method.

    Attributes:
        name (str):
            Required. Full resource name of
            [Product][google.cloud.retail.v2.Product], such as
            ``projects/*/locations/global/catalogs/default_catalog/branches/default_branch/products/some_product_id``.

            If the caller does not have permission to delete the
            [Product][google.cloud.retail.v2.Product], regardless of
            whether or not it exists, a PERMISSION_DENIED error is
            returned.

            If the [Product][google.cloud.retail.v2.Product] to delete
            does not exist, a NOT_FOUND error is returned.

            The [Product][google.cloud.retail.v2.Product] to delete can
            neither be a
            [Product.Type.COLLECTION][google.cloud.retail.v2.Product.Type.COLLECTION]
            [Product][google.cloud.retail.v2.Product] member nor a
            [Product.Type.PRIMARY][google.cloud.retail.v2.Product.Type.PRIMARY]
            [Product][google.cloud.retail.v2.Product] with more than one
            [variants][google.cloud.retail.v2.Product.Type.VARIANT].
            Otherwise, an INVALID_ARGUMENT error is returned.

            All inventory information for the named
            [Product][google.cloud.retail.v2.Product] will be deleted.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListProductsRequest(proto.Message):
    r"""Request message for
    [ProductService.ListProducts][google.cloud.retail.v2.ProductService.ListProducts]
    method.

    Attributes:
        parent (str):
            Required. The parent branch resource name, such as
            ``projects/*/locations/global/catalogs/default_catalog/branches/0``.
            Use ``default_branch`` as the branch ID, to list products
            under the default branch.

            If the caller does not have permission to list
            [Product][google.cloud.retail.v2.Product]s under this
            branch, regardless of whether or not this branch exists, a
            PERMISSION_DENIED error is returned.
        page_size (int):
            Maximum number of [Product][google.cloud.retail.v2.Product]s
            to return. If unspecified, defaults to 100. The maximum
            allowed value is 1000. Values above 1000 will be coerced to
            1000.

            If this field is negative, an INVALID_ARGUMENT error is
            returned.
        page_token (str):
            A page token
            [ListProductsResponse.next_page_token][google.cloud.retail.v2.ListProductsResponse.next_page_token],
            received from a previous
            [ProductService.ListProducts][google.cloud.retail.v2.ProductService.ListProducts]
            call. Provide this to retrieve the subsequent page.

            When paginating, all other parameters provided to
            [ProductService.ListProducts][google.cloud.retail.v2.ProductService.ListProducts]
            must match the call that provided the page token. Otherwise,
            an INVALID_ARGUMENT error is returned.
        filter (str):
            A filter to apply on the list results. Supported features:

            -  List all the products under the parent branch if
               [filter][google.cloud.retail.v2.ListProductsRequest.filter]
               is unset.
            -  List
               [Product.Type.VARIANT][google.cloud.retail.v2.Product.Type.VARIANT]
               [Product][google.cloud.retail.v2.Product]s sharing the
               same
               [Product.Type.PRIMARY][google.cloud.retail.v2.Product.Type.PRIMARY]
               [Product][google.cloud.retail.v2.Product]. For example:
               ``primary_product_id = "some_product_id"``
            -  List [Product][google.cloud.retail.v2.Product]s bundled
               in a
               [Product.Type.COLLECTION][google.cloud.retail.v2.Product.Type.COLLECTION]
               [Product][google.cloud.retail.v2.Product]. For example:
               ``collection_product_id = "some_product_id"``
            -  List [Product][google.cloud.retail.v2.Product]s with a
               partibular type. For example: ``type = "PRIMARY"``
               ``type = "VARIANT"`` ``type = "COLLECTION"``

            If the field is unrecognizable, an INVALID_ARGUMENT error is
            returned.

            If the specified
            [Product.Type.PRIMARY][google.cloud.retail.v2.Product.Type.PRIMARY]
            [Product][google.cloud.retail.v2.Product] or
            [Product.Type.COLLECTION][google.cloud.retail.v2.Product.Type.COLLECTION]
            [Product][google.cloud.retail.v2.Product] does not exist, a
            NOT_FOUND error is returned.
        read_mask (google.protobuf.field_mask_pb2.FieldMask):
            The fields of [Product][google.cloud.retail.v2.Product] to
            return in the responses. If not set or empty, the following
            fields are returned:

            -  [Product.name][google.cloud.retail.v2.Product.name]
            -  [Product.id][google.cloud.retail.v2.Product.id]
            -  [Product.title][google.cloud.retail.v2.Product.title]
            -  [Product.uri][google.cloud.retail.v2.Product.uri]
            -  [Product.images][google.cloud.retail.v2.Product.images]
            -  [Product.price_info][google.cloud.retail.v2.Product.price_info]
            -  [Product.brands][google.cloud.retail.v2.Product.brands]

            If "*" is provided, all fields are returned.
            [Product.name][google.cloud.retail.v2.Product.name] is
            always returned no matter what mask is set.

            If an unsupported or unknown field is provided, an
            INVALID_ARGUMENT error is returned.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=4,
    )
    read_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=5,
        message=field_mask_pb2.FieldMask,
    )


class ListProductsResponse(proto.Message):
    r"""Response message for
    [ProductService.ListProducts][google.cloud.retail.v2.ProductService.ListProducts]
    method.

    Attributes:
        products (MutableSequence[google.cloud.retail_v2.types.Product]):
            The [Product][google.cloud.retail.v2.Product]s.
        next_page_token (str):
            A token that can be sent as
            [ListProductsRequest.page_token][google.cloud.retail.v2.ListProductsRequest.page_token]
            to retrieve the next page. If this field is omitted, there
            are no subsequent pages.
    """

    @property
    def raw_page(self):
        return self

    products: MutableSequence[gcr_product.Product] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gcr_product.Product,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class SetInventoryRequest(proto.Message):
    r"""Request message for
    [ProductService.SetInventory][google.cloud.retail.v2.ProductService.SetInventory]
    method.

    Attributes:
        inventory (google.cloud.retail_v2.types.Product):
            Required. The inventory information to update. The allowable
            fields to update are:

            -  [Product.price_info][google.cloud.retail.v2.Product.price_info]
            -  [Product.availability][google.cloud.retail.v2.Product.availability]
            -  [Product.available_quantity][google.cloud.retail.v2.Product.available_quantity]
            -  [Product.fulfillment_info][google.cloud.retail.v2.Product.fulfillment_info]
               The updated inventory fields must be specified in
               [SetInventoryRequest.set_mask][google.cloud.retail.v2.SetInventoryRequest.set_mask].

            If
            [SetInventoryRequest.inventory.name][google.cloud.retail.v2.Product.name]
            is empty or invalid, an INVALID_ARGUMENT error is returned.

            If the caller does not have permission to update the
            [Product][google.cloud.retail.v2.Product] named in
            [Product.name][google.cloud.retail.v2.Product.name],
            regardless of whether or not it exists, a PERMISSION_DENIED
            error is returned.

            If the [Product][google.cloud.retail.v2.Product] to update
            does not have existing inventory information, the provided
            inventory information will be inserted.

            If the [Product][google.cloud.retail.v2.Product] to update
            has existing inventory information, the provided inventory
            information will be merged while respecting the last update
            time for each inventory field, using the provided or default
            value for
            [SetInventoryRequest.set_time][google.cloud.retail.v2.SetInventoryRequest.set_time].

            The caller can replace place IDs for a subset of fulfillment
            types in the following ways:

            -  Adds "fulfillment_info" in
               [SetInventoryRequest.set_mask][google.cloud.retail.v2.SetInventoryRequest.set_mask]
            -  Specifies only the desired fulfillment types and
               corresponding place IDs to update in
               [SetInventoryRequest.inventory.fulfillment_info][google.cloud.retail.v2.Product.fulfillment_info]

            The caller can clear all place IDs from a subset of
            fulfillment types in the following ways:

            -  Adds "fulfillment_info" in
               [SetInventoryRequest.set_mask][google.cloud.retail.v2.SetInventoryRequest.set_mask]
            -  Specifies only the desired fulfillment types to clear in
               [SetInventoryRequest.inventory.fulfillment_info][google.cloud.retail.v2.Product.fulfillment_info]
            -  Checks that only the desired fulfillment info types have
               empty
               [SetInventoryRequest.inventory.fulfillment_info.place_ids][google.cloud.retail.v2.FulfillmentInfo.place_ids]

            The last update time is recorded for the following inventory
            fields:

            -  [Product.price_info][google.cloud.retail.v2.Product.price_info]
            -  [Product.availability][google.cloud.retail.v2.Product.availability]
            -  [Product.available_quantity][google.cloud.retail.v2.Product.available_quantity]
            -  [Product.fulfillment_info][google.cloud.retail.v2.Product.fulfillment_info]

            If a full overwrite of inventory information while ignoring
            timestamps is needed,
            [ProductService.UpdateProduct][google.cloud.retail.v2.ProductService.UpdateProduct]
            should be invoked instead.
        set_mask (google.protobuf.field_mask_pb2.FieldMask):
            Indicates which inventory fields in the provided
            [Product][google.cloud.retail.v2.Product] to update.

            At least one field must be provided.

            If an unsupported or unknown field is provided, an
            INVALID_ARGUMENT error is returned and the entire update
            will be ignored.
        set_time (google.protobuf.timestamp_pb2.Timestamp):
            The time when the request is issued, used to
            prevent out-of-order updates on inventory fields
            with the last update time recorded. If not
            provided, the internal system time will be used.
        allow_missing (bool):
            If set to true, and the
            [Product][google.cloud.retail.v2.Product] with name
            [Product.name][google.cloud.retail.v2.Product.name] is not
            found, the inventory update will still be processed and
            retained for at most 1 day until the
            [Product][google.cloud.retail.v2.Product] is created. If set
            to false, a NOT_FOUND error is returned if the
            [Product][google.cloud.retail.v2.Product] is not found.
    """

    inventory: gcr_product.Product = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gcr_product.Product,
    )
    set_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )
    set_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )
    allow_missing: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class SetInventoryMetadata(proto.Message):
    r"""Metadata related to the progress of the SetInventory operation.
    Currently empty because there is no meaningful metadata populated
    from the
    [ProductService.SetInventory][google.cloud.retail.v2.ProductService.SetInventory]
    method.

    """


class SetInventoryResponse(proto.Message):
    r"""Response of the SetInventoryRequest. Currently empty because there
    is no meaningful response populated from the
    [ProductService.SetInventory][google.cloud.retail.v2.ProductService.SetInventory]
    method.

    """


class AddFulfillmentPlacesRequest(proto.Message):
    r"""Request message for
    [ProductService.AddFulfillmentPlaces][google.cloud.retail.v2.ProductService.AddFulfillmentPlaces]
    method.

    Attributes:
        product (str):
            Required. Full resource name of
            [Product][google.cloud.retail.v2.Product], such as
            ``projects/*/locations/global/catalogs/default_catalog/branches/default_branch/products/some_product_id``.

            If the caller does not have permission to access the
            [Product][google.cloud.retail.v2.Product], regardless of
            whether or not it exists, a PERMISSION_DENIED error is
            returned.
        type_ (str):
            Required. The fulfillment type, including commonly used
            types (such as pickup in store and same day delivery), and
            custom types.

            Supported values:

            -  "pickup-in-store"
            -  "ship-to-store"
            -  "same-day-delivery"
            -  "next-day-delivery"
            -  "custom-type-1"
            -  "custom-type-2"
            -  "custom-type-3"
            -  "custom-type-4"
            -  "custom-type-5"

            If this field is set to an invalid value other than these,
            an INVALID_ARGUMENT error is returned.

            This field directly corresponds to
            [Product.fulfillment_info.type][google.cloud.retail.v2.FulfillmentInfo.type].
        place_ids (MutableSequence[str]):
            Required. The IDs for this
            [type][google.cloud.retail.v2.AddFulfillmentPlacesRequest.type],
            such as the store IDs for "pickup-in-store" or the region
            IDs for "same-day-delivery" to be added for this
            [type][google.cloud.retail.v2.AddFulfillmentPlacesRequest.type].
            Duplicate IDs will be automatically ignored.

            At least 1 value is required, and a maximum of 2000 values
            are allowed. Each value must be a string with a length limit
            of 10 characters, matching the pattern ``[a-zA-Z0-9_-]+``,
            such as "store1" or "REGION-2". Otherwise, an
            INVALID_ARGUMENT error is returned.

            If the total number of place IDs exceeds 2000 for this
            [type][google.cloud.retail.v2.AddFulfillmentPlacesRequest.type]
            after adding, then the update will be rejected.
        add_time (google.protobuf.timestamp_pb2.Timestamp):
            The time when the fulfillment updates are
            issued, used to prevent out-of-order updates on
            fulfillment information. If not provided, the
            internal system time will be used.
        allow_missing (bool):
            If set to true, and the
            [Product][google.cloud.retail.v2.Product] is not found, the
            fulfillment information will still be processed and retained
            for at most 1 day and processed once the
            [Product][google.cloud.retail.v2.Product] is created. If set
            to false, a NOT_FOUND error is returned if the
            [Product][google.cloud.retail.v2.Product] is not found.
    """

    product: str = proto.Field(
        proto.STRING,
        number=1,
    )
    type_: str = proto.Field(
        proto.STRING,
        number=2,
    )
    place_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )
    add_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timestamp_pb2.Timestamp,
    )
    allow_missing: bool = proto.Field(
        proto.BOOL,
        number=5,
    )


class AddFulfillmentPlacesMetadata(proto.Message):
    r"""Metadata related to the progress of the AddFulfillmentPlaces
    operation. Currently empty because there is no meaningful metadata
    populated from the
    [ProductService.AddFulfillmentPlaces][google.cloud.retail.v2.ProductService.AddFulfillmentPlaces]
    method.

    """


class AddFulfillmentPlacesResponse(proto.Message):
    r"""Response of the AddFulfillmentPlacesRequest. Currently empty because
    there is no meaningful response populated from the
    [ProductService.AddFulfillmentPlaces][google.cloud.retail.v2.ProductService.AddFulfillmentPlaces]
    method.

    """


class AddLocalInventoriesRequest(proto.Message):
    r"""Request message for
    [ProductService.AddLocalInventories][google.cloud.retail.v2.ProductService.AddLocalInventories]
    method.

    Attributes:
        product (str):
            Required. Full resource name of
            [Product][google.cloud.retail.v2.Product], such as
            ``projects/*/locations/global/catalogs/default_catalog/branches/default_branch/products/some_product_id``.

            If the caller does not have permission to access the
            [Product][google.cloud.retail.v2.Product], regardless of
            whether or not it exists, a PERMISSION_DENIED error is
            returned.
        local_inventories (MutableSequence[google.cloud.retail_v2.types.LocalInventory]):
            Required. A list of inventory information at
            difference places. Each place is identified by
            its place ID. At most 3000 inventories are
            allowed per request.
        add_mask (google.protobuf.field_mask_pb2.FieldMask):
            Indicates which inventory fields in the provided list of
            [LocalInventory][google.cloud.retail.v2.LocalInventory] to
            update. The field is updated to the provided value.

            If a field is set while the place does not have a previous
            local inventory, the local inventory at that store is
            created.

            If a field is set while the value of that field is not
            provided, the original field value, if it exists, is
            deleted.

            If the mask is not set or set with empty paths, all
            inventory fields will be updated.

            If an unsupported or unknown field is provided, an
            INVALID_ARGUMENT error is returned and the entire update
            will be ignored.
        add_time (google.protobuf.timestamp_pb2.Timestamp):
            The time when the inventory updates are
            issued. Used to prevent out-of-order updates on
            local inventory fields. If not provided, the
            internal system time will be used.
        allow_missing (bool):
            If set to true, and the
            [Product][google.cloud.retail.v2.Product] is not found, the
            local inventory will still be processed and retained for at
            most 1 day and processed once the
            [Product][google.cloud.retail.v2.Product] is created. If set
            to false, a NOT_FOUND error is returned if the
            [Product][google.cloud.retail.v2.Product] is not found.
    """

    product: str = proto.Field(
        proto.STRING,
        number=1,
    )
    local_inventories: MutableSequence[common.LocalInventory] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=common.LocalInventory,
    )
    add_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=4,
        message=field_mask_pb2.FieldMask,
    )
    add_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    allow_missing: bool = proto.Field(
        proto.BOOL,
        number=6,
    )


class AddLocalInventoriesMetadata(proto.Message):
    r"""Metadata related to the progress of the AddLocalInventories
    operation. Currently empty because there is no meaningful metadata
    populated from the
    [ProductService.AddLocalInventories][google.cloud.retail.v2.ProductService.AddLocalInventories]
    method.

    """


class AddLocalInventoriesResponse(proto.Message):
    r"""Response of the
    [ProductService.AddLocalInventories][google.cloud.retail.v2.ProductService.AddLocalInventories]
    API. Currently empty because there is no meaningful response
    populated from the
    [ProductService.AddLocalInventories][google.cloud.retail.v2.ProductService.AddLocalInventories]
    method.

    """


class RemoveLocalInventoriesRequest(proto.Message):
    r"""Request message for
    [ProductService.RemoveLocalInventories][google.cloud.retail.v2.ProductService.RemoveLocalInventories]
    method.

    Attributes:
        product (str):
            Required. Full resource name of
            [Product][google.cloud.retail.v2.Product], such as
            ``projects/*/locations/global/catalogs/default_catalog/branches/default_branch/products/some_product_id``.

            If the caller does not have permission to access the
            [Product][google.cloud.retail.v2.Product], regardless of
            whether or not it exists, a PERMISSION_DENIED error is
            returned.
        place_ids (MutableSequence[str]):
            Required. A list of place IDs to have their
            inventory deleted. At most 3000 place IDs are
            allowed per request.
        remove_time (google.protobuf.timestamp_pb2.Timestamp):
            The time when the inventory deletions are
            issued. Used to prevent out-of-order updates and
            deletions on local inventory fields. If not
            provided, the internal system time will be used.
        allow_missing (bool):
            If set to true, and the
            [Product][google.cloud.retail.v2.Product] is not found, the
            local inventory removal request will still be processed and
            retained for at most 1 day and processed once the
            [Product][google.cloud.retail.v2.Product] is created. If set
            to false, a NOT_FOUND error is returned if the
            [Product][google.cloud.retail.v2.Product] is not found.
    """

    product: str = proto.Field(
        proto.STRING,
        number=1,
    )
    place_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=2,
    )
    remove_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    allow_missing: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class RemoveLocalInventoriesMetadata(proto.Message):
    r"""Metadata related to the progress of the RemoveLocalInventories
    operation. Currently empty because there is no meaningful metadata
    populated from the
    [ProductService.RemoveLocalInventories][google.cloud.retail.v2.ProductService.RemoveLocalInventories]
    method.

    """


class RemoveLocalInventoriesResponse(proto.Message):
    r"""Response of the
    [ProductService.RemoveLocalInventories][google.cloud.retail.v2.ProductService.RemoveLocalInventories]
    API. Currently empty because there is no meaningful response
    populated from the
    [ProductService.RemoveLocalInventories][google.cloud.retail.v2.ProductService.RemoveLocalInventories]
    method.

    """


class RemoveFulfillmentPlacesRequest(proto.Message):
    r"""Request message for
    [ProductService.RemoveFulfillmentPlaces][google.cloud.retail.v2.ProductService.RemoveFulfillmentPlaces]
    method.

    Attributes:
        product (str):
            Required. Full resource name of
            [Product][google.cloud.retail.v2.Product], such as
            ``projects/*/locations/global/catalogs/default_catalog/branches/default_branch/products/some_product_id``.

            If the caller does not have permission to access the
            [Product][google.cloud.retail.v2.Product], regardless of
            whether or not it exists, a PERMISSION_DENIED error is
            returned.
        type_ (str):
            Required. The fulfillment type, including commonly used
            types (such as pickup in store and same day delivery), and
            custom types.

            Supported values:

            -  "pickup-in-store"
            -  "ship-to-store"
            -  "same-day-delivery"
            -  "next-day-delivery"
            -  "custom-type-1"
            -  "custom-type-2"
            -  "custom-type-3"
            -  "custom-type-4"
            -  "custom-type-5"

            If this field is set to an invalid value other than these,
            an INVALID_ARGUMENT error is returned.

            This field directly corresponds to
            [Product.fulfillment_info.type][google.cloud.retail.v2.FulfillmentInfo.type].
        place_ids (MutableSequence[str]):
            Required. The IDs for this
            [type][google.cloud.retail.v2.RemoveFulfillmentPlacesRequest.type],
            such as the store IDs for "pickup-in-store" or the region
            IDs for "same-day-delivery", to be removed for this
            [type][google.cloud.retail.v2.RemoveFulfillmentPlacesRequest.type].

            At least 1 value is required, and a maximum of 2000 values
            are allowed. Each value must be a string with a length limit
            of 10 characters, matching the pattern ``[a-zA-Z0-9_-]+``,
            such as "store1" or "REGION-2". Otherwise, an
            INVALID_ARGUMENT error is returned.
        remove_time (google.protobuf.timestamp_pb2.Timestamp):
            The time when the fulfillment updates are
            issued, used to prevent out-of-order updates on
            fulfillment information. If not provided, the
            internal system time will be used.
        allow_missing (bool):
            If set to true, and the
            [Product][google.cloud.retail.v2.Product] is not found, the
            fulfillment information will still be processed and retained
            for at most 1 day and processed once the
            [Product][google.cloud.retail.v2.Product] is created. If set
            to false, a NOT_FOUND error is returned if the
            [Product][google.cloud.retail.v2.Product] is not found.
    """

    product: str = proto.Field(
        proto.STRING,
        number=1,
    )
    type_: str = proto.Field(
        proto.STRING,
        number=2,
    )
    place_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )
    remove_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timestamp_pb2.Timestamp,
    )
    allow_missing: bool = proto.Field(
        proto.BOOL,
        number=5,
    )


class RemoveFulfillmentPlacesMetadata(proto.Message):
    r"""Metadata related to the progress of the RemoveFulfillmentPlaces
    operation. Currently empty because there is no meaningful metadata
    populated from the
    [ProductService.RemoveFulfillmentPlaces][google.cloud.retail.v2.ProductService.RemoveFulfillmentPlaces]
    method.

    """


class RemoveFulfillmentPlacesResponse(proto.Message):
    r"""Response of the RemoveFulfillmentPlacesRequest. Currently empty
    because there is no meaningful response populated from the
    [ProductService.RemoveFulfillmentPlaces][google.cloud.retail.v2.ProductService.RemoveFulfillmentPlaces]
    method.

    """


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import struct_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.retail_v2alpha.types import common
from google.cloud.retail_v2alpha.types import product as gcr_product

__protobuf__ = proto.module(
    package="google.cloud.retail.v2alpha",
    manifest={
        "SearchRequest",
        "SearchResponse",
        "ExperimentInfo",
    },
)


class SearchRequest(proto.Message):
    r"""Request message for
    [SearchService.Search][google.cloud.retail.v2alpha.SearchService.Search]
    method.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        placement (str):
            Required. The resource name of the Retail Search serving
            config, such as
            ``projects/*/locations/global/catalogs/default_catalog/servingConfigs/default_serving_config``
            or the name of the legacy placement resource, such as
            ``projects/*/locations/global/catalogs/default_catalog/placements/default_search``.
            This field is used to identify the serving config name and
            the set of models that will be used to make the search.
        branch (str):
            The branch resource name, such as
            ``projects/*/locations/global/catalogs/default_catalog/branches/0``.

            Use "default_branch" as the branch ID or leave this field
            empty, to search products under the default branch.
        query (str):
            Raw search query.

            If this field is empty, the request is considered a category
            browsing request and returned results are based on
            [filter][google.cloud.retail.v2alpha.SearchRequest.filter]
            and
            [page_categories][google.cloud.retail.v2alpha.SearchRequest.page_categories].
        visitor_id (str):
            Required. A unique identifier for tracking visitors. For
            example, this could be implemented with an HTTP cookie,
            which should be able to uniquely identify a visitor on a
            single device. This unique identifier should not change if
            the visitor logs in or out of the website.

            This should be the same identifier as
            [UserEvent.visitor_id][google.cloud.retail.v2alpha.UserEvent.visitor_id].

            The field must be a UTF-8 encoded string with a length limit
            of 128 characters. Otherwise, an INVALID_ARGUMENT error is
            returned.
        user_info (google.cloud.retail_v2alpha.types.UserInfo):
            User information.
        page_size (int):
            Maximum number of
            [Product][google.cloud.retail.v2alpha.Product]s to return.
            If unspecified, defaults to a reasonable value. The maximum
            allowed value is 120. Values above 120 will be coerced to
            120.

            If this field is negative, an INVALID_ARGUMENT is returned.
        page_token (str):
            A page token
            [SearchResponse.next_page_token][google.cloud.retail.v2alpha.SearchResponse.next_page_token],
            received from a previous
            [SearchService.Search][google.cloud.retail.v2alpha.SearchService.Search]
            call. Provide this to retrieve the subsequent page.

            When paginating, all other parameters provided to
            [SearchService.Search][google.cloud.retail.v2alpha.SearchService.Search]
            must match the call that provided the page token. Otherwise,
            an INVALID_ARGUMENT error is returned.
        offset (int):
            A 0-indexed integer that specifies the current offset (that
            is, starting result location, amongst the
            [Product][google.cloud.retail.v2alpha.Product]s deemed by
            the API as relevant) in search results. This field is only
            considered if
            [page_token][google.cloud.retail.v2alpha.SearchRequest.page_token]
            is unset.

            If this field is negative, an INVALID_ARGUMENT is returned.
        filter (str):
            The filter syntax consists of an expression language for
            constructing a predicate from one or more fields of the
            products being filtered. Filter expression is
            case-sensitive. See more details at this `user
            guide <https://cloud.google.com/retail/docs/filter-and-order#filter>`__.

            If this field is unrecognizable, an INVALID_ARGUMENT is
            returned.
        canonical_filter (str):
            The default filter that is applied when a user performs a
            search without checking any filters on the search page.

            The filter applied to every search request when quality
            improvement such as query expansion is needed. For example,
            if a query does not have enough results, an expanded query
            with
            [SearchRequest.canonical_filter][google.cloud.retail.v2alpha.SearchRequest.canonical_filter]
            will be returned as a supplement of the original query. This
            field is strongly recommended to achieve high search
            quality.

            See
            [SearchRequest.filter][google.cloud.retail.v2alpha.SearchRequest.filter]
            for more details about filter syntax.
        order_by (str):
            The order in which products are returned. Products can be
            ordered by a field in an
            [Product][google.cloud.retail.v2alpha.Product] object. Leave
            it unset if ordered by relevance. OrderBy expression is
            case-sensitive. See more details at this `user
            guide <https://cloud.google.com/retail/docs/filter-and-order#order>`__.

            If this field is unrecognizable, an INVALID_ARGUMENT is
            returned.
        facet_specs (MutableSequence[google.cloud.retail_v2alpha.types.SearchRequest.FacetSpec]):
            Facet specifications for faceted search. If empty, no facets
            are returned.

            A maximum of 200 values are allowed. Otherwise, an
            INVALID_ARGUMENT error is returned.
        dynamic_facet_spec (google.cloud.retail_v2alpha.types.SearchRequest.DynamicFacetSpec):
            Deprecated. Refer to
            https://cloud.google.com/retail/docs/configs#dynamic
            to enable dynamic facets. Do not set this field.

            The specification for dynamically generated
            facets. Notice that only textual facets can be
            dynamically generated.
        boost_spec (google.cloud.retail_v2alpha.types.SearchRequest.BoostSpec):
            Boost specification to boost certain products. See more
            details at this `user
            guide <https://cloud.google.com/retail/docs/boosting>`__.

            Notice that if both
            [ServingConfig.boost_control_ids][google.cloud.retail.v2alpha.ServingConfig.boost_control_ids]
            and
            [SearchRequest.boost_spec][google.cloud.retail.v2alpha.SearchRequest.boost_spec]
            are set, the boost conditions from both places are
            evaluated. If a search request matches multiple boost
            conditions, the final boost score is equal to the sum of the
            boost scores from all matched boost conditions.
        query_expansion_spec (google.cloud.retail_v2alpha.types.SearchRequest.QueryExpansionSpec):
            The query expansion specification that specifies the
            conditions under which query expansion will occur. See more
            details at this `user
            guide <https://cloud.google.com/retail/docs/result-size#query_expansion>`__.
        relevance_threshold (google.cloud.retail_v2alpha.types.SearchRequest.RelevanceThreshold):
            The relevance threshold of the search results.

            Defaults to
            [RelevanceThreshold.HIGH][google.cloud.retail.v2alpha.SearchRequest.RelevanceThreshold.HIGH],
            which means only the most relevant results are shown, and
            the least number of results are returned. See more details
            at this `user
            guide <https://cloud.google.com/retail/docs/result-size#relevance_thresholding>`__.
        variant_rollup_keys (MutableSequence[str]):
            The keys to fetch and rollup the matching
            [variant][google.cloud.retail.v2alpha.Product.Type.VARIANT]
            [Product][google.cloud.retail.v2alpha.Product]s attributes,
            [FulfillmentInfo][google.cloud.retail.v2alpha.FulfillmentInfo]
            or
            [LocalInventory][google.cloud.retail.v2alpha.LocalInventory]s
            attributes. The attributes from all the matching
            [variant][google.cloud.retail.v2alpha.Product.Type.VARIANT]
            [Product][google.cloud.retail.v2alpha.Product]s or
            [LocalInventory][google.cloud.retail.v2alpha.LocalInventory]s
            are merged and de-duplicated. Notice that rollup attributes
            will lead to extra query latency. Maximum number of keys is
            30.

            For
            [FulfillmentInfo][google.cloud.retail.v2alpha.FulfillmentInfo],
            a fulfillment type and a fulfillment ID must be provided in
            the format of "fulfillmentType.fulfillmentId". E.g., in
            "pickupInStore.store123", "pickupInStore" is fulfillment
            type and "store123" is the store ID.

            Supported keys are:

            -  colorFamilies
            -  price
            -  originalPrice
            -  discount
            -  variantId
            -  inventory(place_id,price)
            -  inventory(place_id,original_price)
            -  inventory(place_id,attributes.key), where key is any key
               in the
               [Product.local_inventories.attributes][google.cloud.retail.v2alpha.LocalInventory.attributes]
               map.
            -  attributes.key, where key is any key in the
               [Product.attributes][google.cloud.retail.v2alpha.Product.attributes]
               map.
            -  pickupInStore.id, where id is any
               [FulfillmentInfo.place_ids][google.cloud.retail.v2alpha.FulfillmentInfo.place_ids]
               for
               [FulfillmentInfo.type][google.cloud.retail.v2alpha.FulfillmentInfo.type]
               "pickup-in-store".
            -  shipToStore.id, where id is any
               [FulfillmentInfo.place_ids][google.cloud.retail.v2alpha.FulfillmentInfo.place_ids]
               for
               [FulfillmentInfo.type][google.cloud.retail.v2alpha.FulfillmentInfo.type]
               "ship-to-store".
            -  sameDayDelivery.id, where id is any
               [FulfillmentInfo.place_ids][google.cloud.retail.v2alpha.FulfillmentInfo.place_ids]
               for
               [FulfillmentInfo.type][google.cloud.retail.v2alpha.FulfillmentInfo.type]
               "same-day-delivery".
            -  nextDayDelivery.id, where id is any
               [FulfillmentInfo.place_ids][google.cloud.retail.v2alpha.FulfillmentInfo.place_ids]
               for
               [FulfillmentInfo.type][google.cloud.retail.v2alpha.FulfillmentInfo.type]
               "next-day-delivery".
            -  customFulfillment1.id, where id is any
               [FulfillmentInfo.place_ids][google.cloud.retail.v2alpha.FulfillmentInfo.place_ids]
               for
               [FulfillmentInfo.type][google.cloud.retail.v2alpha.FulfillmentInfo.type]
               "custom-type-1".
            -  customFulfillment2.id, where id is any
               [FulfillmentInfo.place_ids][google.cloud.retail.v2alpha.FulfillmentInfo.place_ids]
               for
               [FulfillmentInfo.type][google.cloud.retail.v2alpha.FulfillmentInfo.type]
               "custom-type-2".
            -  customFulfillment3.id, where id is any
               [FulfillmentInfo.place_ids][google.cloud.retail.v2alpha.FulfillmentInfo.place_ids]
               for
               [FulfillmentInfo.type][google.cloud.retail.v2alpha.FulfillmentInfo.type]
               "custom-type-3".
            -  customFulfillment4.id, where id is any
               [FulfillmentInfo.place_ids][google.cloud.retail.v2alpha.FulfillmentInfo.place_ids]
               for
               [FulfillmentInfo.type][google.cloud.retail.v2alpha.FulfillmentInfo.type]
               "custom-type-4".
            -  customFulfillment5.id, where id is any
               [FulfillmentInfo.place_ids][google.cloud.retail.v2alpha.FulfillmentInfo.place_ids]
               for
               [FulfillmentInfo.type][google.cloud.retail.v2alpha.FulfillmentInfo.type]
               "custom-type-5".

            If this field is set to an invalid value other than these,
            an INVALID_ARGUMENT error is returned.
        page_categories (MutableSequence[str]):
            The categories associated with a category page. Must be set
            for category navigation queries to achieve good search
            quality. The format should be the same as
            [UserEvent.page_categories][google.cloud.retail.v2alpha.UserEvent.page_categories];

            To represent full path of category, use '>' sign to separate
            different hierarchies. If '>' is part of the category name,
            replace it with other character(s).

            Category pages include special pages such as sales or
            promotions. For instance, a special sale page may have the
            category hierarchy: "pageCategories" : ["Sales > 2017 Black
            Friday Deals"].
        search_mode (google.cloud.retail_v2alpha.types.SearchRequest.SearchMode):
            The search mode of the search request. If not
            specified, a single search request triggers both
            product search and faceted search.
        personalization_spec (google.cloud.retail_v2alpha.types.SearchRequest.PersonalizationSpec):
            The specification for personalization.

            Notice that if both
            [ServingConfig.personalization_spec][google.cloud.retail.v2alpha.ServingConfig.personalization_spec]
            and
            [SearchRequest.personalization_spec][google.cloud.retail.v2alpha.SearchRequest.personalization_spec]
            are set.
            [SearchRequest.personalization_spec][google.cloud.retail.v2alpha.SearchRequest.personalization_spec]
            will override
            [ServingConfig.personalization_spec][google.cloud.retail.v2alpha.ServingConfig.personalization_spec].
        labels (MutableMapping[str, str]):
            The labels applied to a resource must meet the following
            requirements:

            -  Each resource can have multiple labels, up to a maximum
               of 64.
            -  Each label must be a key-value pair.
            -  Keys have a minimum length of 1 character and a maximum
               length of 63 characters and cannot be empty. Values can
               be empty and have a maximum length of 63 characters.
            -  Keys and values can contain only lowercase letters,
               numeric characters, underscores, and dashes. All
               characters must use UTF-8 encoding, and international
               characters are allowed.
            -  The key portion of a label must be unique. However, you
               can use the same key with multiple resources.
            -  Keys must start with a lowercase letter or international
               character.

            See `Google Cloud
            Document <https://cloud.google.com/resource-manager/docs/creating-managing-labels#requirements>`__
            for more details.
        spell_correction_spec (google.cloud.retail_v2alpha.types.SearchRequest.SpellCorrectionSpec):
            The spell correction specification that
            specifies the mode under which spell correction
            will take effect.

            This field is a member of `oneof`_ ``_spell_correction_spec``.
        entity (str):
            The entity for customers that may run multiple different
            entities, domains, sites or regions, for example,
            ``Google US``, ``Google Ads``, ``Waymo``, ``google.com``,
            ``youtube.com``, etc. If this is set, it should be exactly
            matched with
            [UserEvent.entity][google.cloud.retail.v2alpha.UserEvent.entity]
            to get search results boosted by entity.
    """

    class RelevanceThreshold(proto.Enum):
        r"""The relevance threshold of the search results. The higher
        relevance threshold is, the higher relevant results are shown
        and the less number of results are returned.

        Values:
            RELEVANCE_THRESHOLD_UNSPECIFIED (0):
                Default value. In this case, server behavior defaults to
                [RelevanceThreshold.HIGH][google.cloud.retail.v2alpha.SearchRequest.RelevanceThreshold.HIGH].
            HIGH (1):
                High relevance threshold.
            MEDIUM (2):
                Medium relevance threshold.
            LOW (3):
                Low relevance threshold.
            LOWEST (4):
                Lowest relevance threshold.
        """
        RELEVANCE_THRESHOLD_UNSPECIFIED = 0
        HIGH = 1
        MEDIUM = 2
        LOW = 3
        LOWEST = 4

    class SearchMode(proto.Enum):
        r"""The search mode of each search request.

        Values:
            SEARCH_MODE_UNSPECIFIED (0):
                Default value. In this case both product search and faceted
                search will be performed. Both
                [SearchResponse.SearchResult][google.cloud.retail.v2alpha.SearchResponse.SearchResult]
                and
                [SearchResponse.Facet][google.cloud.retail.v2alpha.SearchResponse.Facet]
                will be returned.
            PRODUCT_SEARCH_ONLY (1):
                Only product search will be performed. The faceted search
                will be disabled.

                Only
                [SearchResponse.SearchResult][google.cloud.retail.v2alpha.SearchResponse.SearchResult]
                will be returned.
                [SearchResponse.Facet][google.cloud.retail.v2alpha.SearchResponse.Facet]
                will not be returned, even if
                [SearchRequest.facet_specs][google.cloud.retail.v2alpha.SearchRequest.facet_specs]
                or
                [SearchRequest.dynamic_facet_spec][google.cloud.retail.v2alpha.SearchRequest.dynamic_facet_spec]
                is set.
            FACETED_SEARCH_ONLY (2):
                Only faceted search will be performed. The product search
                will be disabled.

                When in this mode, one or both of
                [SearchRequest.facet_specs][google.cloud.retail.v2alpha.SearchRequest.facet_specs]
                and
                [SearchRequest.dynamic_facet_spec][google.cloud.retail.v2alpha.SearchRequest.dynamic_facet_spec]
                should be set. Otherwise, an INVALID_ARGUMENT error is
                returned. Only
                [SearchResponse.Facet][google.cloud.retail.v2alpha.SearchResponse.Facet]
                will be returned.
                [SearchResponse.SearchResult][google.cloud.retail.v2alpha.SearchResponse.SearchResult]
                will not be returned.
        """
        SEARCH_MODE_UNSPECIFIED = 0
        PRODUCT_SEARCH_ONLY = 1
        FACETED_SEARCH_ONLY = 2

    class FacetSpec(proto.Message):
        r"""A facet specification to perform faceted search.

        Attributes:
            facet_key (google.cloud.retail_v2alpha.types.SearchRequest.FacetSpec.FacetKey):
                Required. The facet key specification.
            limit (int):
                Maximum of facet values that should be returned for this
                facet. If unspecified, defaults to 50. The maximum allowed
                value is 300. Values above 300 will be coerced to 300.

                If this field is negative, an INVALID_ARGUMENT is returned.
            excluded_filter_keys (MutableSequence[str]):
                List of keys to exclude when faceting.

                By default,
                [FacetKey.key][google.cloud.retail.v2alpha.SearchRequest.FacetSpec.FacetKey.key]
                is not excluded from the filter unless it is listed in this
                field.

                Listing a facet key in this field allows its values to
                appear as facet results, even when they are filtered out of
                search results. Using this field does not affect what search
                results are returned.

                For example, suppose there are 100 products with the color
                facet "Red" and 200 products with the color facet "Blue". A
                query containing the filter "colorFamilies:ANY("Red")" and
                having "colorFamilies" as
                [FacetKey.key][google.cloud.retail.v2alpha.SearchRequest.FacetSpec.FacetKey.key]
                would by default return only "Red" products in the search
                results, and also return "Red" with count 100 as the only
                color facet. Although there are also blue products
                available, "Blue" would not be shown as an available facet
                value.

                If "colorFamilies" is listed in "excludedFilterKeys", then
                the query returns the facet values "Red" with count 100 and
                "Blue" with count 200, because the "colorFamilies" key is
                now excluded from the filter. Because this field doesn't
                affect search results, the search results are still
                correctly filtered to return only "Red" products.

                A maximum of 100 values are allowed. Otherwise, an
                INVALID_ARGUMENT error is returned.
            enable_dynamic_position (bool):
                Enables dynamic position for this facet. If set to true, the
                position of this facet among all facets in the response is
                determined by Google Retail Search. It will be ordered
                together with dynamic facets if dynamic facets is enabled.
                If set to false, the position of this facet in the response
                will be the same as in the request, and it will be ranked
                before the facets with dynamic position enable and all
                dynamic facets.

                For example, you may always want to have rating facet
                returned in the response, but it's not necessarily to always
                display the rating facet at the top. In that case, you can
                set enable_dynamic_position to true so that the position of
                rating facet in response will be determined by Google Retail
                Search.

                Another example, assuming you have the following facets in
                the request:

                -  "rating", enable_dynamic_position = true

                -  "price", enable_dynamic_position = false

                -  "brands", enable_dynamic_position = false

                And also you have a dynamic facets enable, which will
                generate a facet 'gender'. Then the final order of the
                facets in the response can be ("price", "brands", "rating",
                "gender") or ("price", "brands", "gender", "rating") depends
                on how Google Retail Search orders "gender" and "rating"
                facets. However, notice that "price" and "brands" will
                always be ranked at 1st and 2nd position since their
                enable_dynamic_position are false.
        """

        class FacetKey(proto.Message):
            r"""Specifies how a facet is computed.

            Attributes:
                key (str):
                    Required. Supported textual and numerical facet keys in
                    [Product][google.cloud.retail.v2alpha.Product] object, over
                    which the facet values are computed. Facet key is
                    case-sensitive.

                    Allowed facet keys when
                    [FacetKey.query][google.cloud.retail.v2alpha.SearchRequest.FacetSpec.FacetKey.query]
                    is not specified:

                    -  textual_field =

                       -  "brands"
                       -  "categories"
                       -  "genders"
                       -  "ageGroups"
                       -  "availability"
                       -  "colorFamilies"
                       -  "colors"
                       -  "sizes"
                       -  "materials"
                       -  "patterns"
                       -  "conditions"
                       -  "attributes.key"
                       -  "pickupInStore"
                       -  "shipToStore"
                       -  "sameDayDelivery"
                       -  "nextDayDelivery"
                       -  "customFulfillment1"
                       -  "customFulfillment2"
                       -  "customFulfillment3"
                       -  "customFulfillment4"
                       -  "customFulfillment5"
                       -  "inventory(place_id,attributes.key)"

                    -  numerical_field =

                       -  "price"
                       -  "discount"
                       -  "rating"
                       -  "ratingCount"
                       -  "attributes.key"
                       -  "inventory(place_id,price)"
                       -  "inventory(place_id,original_price)"
                       -  "inventory(place_id,attributes.key)".
                intervals (MutableSequence[google.cloud.retail_v2alpha.types.Interval]):
                    Set only if values should be bucketized into
                    intervals. Must be set for facets with numerical
                    values. Must not be set for facet with text
                    values. Maximum number of intervals is 40.

                    For all numerical facet keys that appear in the
                    list of products from the catalog, the
                    percentiles 0, 10, 30, 50, 70, 90 and 100 are
                    computed from their distribution weekly. If the
                    model assigns a high score to a numerical facet
                    key and its intervals are not specified in the
                    search request, these percentiles will become
                    the bounds for its intervals and will be
                    returned in the response. If the facet key
                    intervals are specified in the request, then the
                    specified intervals will be returned instead.
                restricted_values (MutableSequence[str]):
                    Only get facet for the given restricted values. For example,
                    when using "pickupInStore" as key and set restricted values
                    to ["store123", "store456"], only facets for "store123" and
                    "store456" are returned. Only supported on predefined
                    textual fields, custom textual attributes and fulfillments.
                    Maximum is 20.

                    Must be set for the fulfillment facet keys:

                    -  pickupInStore

                    -  shipToStore

                    -  sameDayDelivery

                    -  nextDayDelivery

                    -  customFulfillment1

                    -  customFulfillment2

                    -  customFulfillment3

                    -  customFulfillment4

                    -  customFulfillment5
                prefixes (MutableSequence[str]):
                    Only get facet values that start with the
                    given string prefix. For example, suppose
                    "categories" has three values "Women > Shoe",
                    "Women > Dress" and "Men > Shoe". If set
                    "prefixes" to "Women", the "categories" facet
                    will give only "Women > Shoe" and "Women >
                    Dress". Only supported on textual fields.
                    Maximum is 10.
                contains (MutableSequence[str]):
                    Only get facet values that contains the given
                    strings. For example, suppose "categories" has
                    three values "Women > Shoe", "Women > Dress" and
                    "Men > Shoe". If set "contains" to "Shoe", the
                    "categories" facet will give only "Women > Shoe"
                    and "Men > Shoe". Only supported on textual
                    fields. Maximum is 10.
                case_insensitive (bool):
                    True to make facet keys case insensitive when
                    getting faceting values with prefixes or
                    contains; false otherwise.
                order_by (str):
                    The order in which
                    [SearchResponse.Facet.values][google.cloud.retail.v2alpha.SearchResponse.Facet.values]
                    are returned.

                    Allowed values are:

                    -  "count desc", which means order by
                       [SearchResponse.Facet.values.count][google.cloud.retail.v2alpha.SearchResponse.Facet.FacetValue.count]
                       descending.

                    -  "value desc", which means order by
                       [SearchResponse.Facet.values.value][google.cloud.retail.v2alpha.SearchResponse.Facet.FacetValue.value]
                       descending. Only applies to textual facets.

                    If not set, textual values are sorted in `natural
                    order <https://en.wikipedia.org/wiki/Natural_sort_order>`__;
                    numerical intervals are sorted in the order given by
                    [FacetSpec.FacetKey.intervals][google.cloud.retail.v2alpha.SearchRequest.FacetSpec.FacetKey.intervals];
                    [FulfillmentInfo.place_ids][google.cloud.retail.v2alpha.FulfillmentInfo.place_ids]
                    are sorted in the order given by
                    [FacetSpec.FacetKey.restricted_values][google.cloud.retail.v2alpha.SearchRequest.FacetSpec.FacetKey.restricted_values].
                query (str):
                    The query that is used to compute facet for the given facet
                    key. When provided, it will override the default behavior of
                    facet computation. The query syntax is the same as a filter
                    expression. See
                    [SearchRequest.filter][google.cloud.retail.v2alpha.SearchRequest.filter]
                    for detail syntax and limitations. Notice that there is no
                    limitation on
                    [FacetKey.key][google.cloud.retail.v2alpha.SearchRequest.FacetSpec.FacetKey.key]
                    when query is specified.

                    In the response,
                    [SearchResponse.Facet.values.value][google.cloud.retail.v2alpha.SearchResponse.Facet.FacetValue.value]
                    will be always "1" and
                    [SearchResponse.Facet.values.count][google.cloud.retail.v2alpha.SearchResponse.Facet.FacetValue.count]
                    will be the number of results that match the query.

                    For example, you can set a customized facet for
                    "shipToStore", where
                    [FacetKey.key][google.cloud.retail.v2alpha.SearchRequest.FacetSpec.FacetKey.key]
                    is "customizedShipToStore", and
                    [FacetKey.query][google.cloud.retail.v2alpha.SearchRequest.FacetSpec.FacetKey.query]
                    is "availability: ANY("IN_STOCK") AND shipToStore:
                    ANY("123")". Then the facet will count the products that are
                    both in stock and ship to store "123".
                return_min_max (bool):
                    Returns the min and max value for each
                    numerical facet intervals. Ignored for textual
                    facets.
            """

            key: str = proto.Field(
                proto.STRING,
                number=1,
            )
            intervals: MutableSequence[common.Interval] = proto.RepeatedField(
                proto.MESSAGE,
                number=2,
                message=common.Interval,
            )
            restricted_values: MutableSequence[str] = proto.RepeatedField(
                proto.STRING,
                number=3,
            )
            prefixes: MutableSequence[str] = proto.RepeatedField(
                proto.STRING,
                number=8,
            )
            contains: MutableSequence[str] = proto.RepeatedField(
                proto.STRING,
                number=9,
            )
            case_insensitive: bool = proto.Field(
                proto.BOOL,
                number=10,
            )
            order_by: str = proto.Field(
                proto.STRING,
                number=4,
            )
            query: str = proto.Field(
                proto.STRING,
                number=5,
            )
            return_min_max: bool = proto.Field(
                proto.BOOL,
                number=11,
            )

        facet_key: "SearchRequest.FacetSpec.FacetKey" = proto.Field(
            proto.MESSAGE,
            number=1,
            message="SearchRequest.FacetSpec.FacetKey",
        )
        limit: int = proto.Field(
            proto.INT32,
            number=2,
        )
        excluded_filter_keys: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=3,
        )
        enable_dynamic_position: bool = proto.Field(
            proto.BOOL,
            number=4,
        )

    class DynamicFacetSpec(proto.Message):
        r"""The specifications of dynamically generated facets.

        Attributes:
            mode (google.cloud.retail_v2alpha.types.SearchRequest.DynamicFacetSpec.Mode):
                Mode of the DynamicFacet feature. Defaults to
                [Mode.DISABLED][google.cloud.retail.v2alpha.SearchRequest.DynamicFacetSpec.Mode.DISABLED]
                if it's unset.
        """

        class Mode(proto.Enum):
            r"""Enum to control DynamicFacet mode

            Values:
                MODE_UNSPECIFIED (0):
                    Default value.
                DISABLED (1):
                    Disable Dynamic Facet.
                ENABLED (2):
                    Automatic mode built by Google Retail Search.
            """
            MODE_UNSPECIFIED = 0
            DISABLED = 1
            ENABLED = 2

        mode: "SearchRequest.DynamicFacetSpec.Mode" = proto.Field(
            proto.ENUM,
            number=1,
            enum="SearchRequest.DynamicFacetSpec.Mode",
        )

    class BoostSpec(proto.Message):
        r"""Boost specification to boost certain items.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            condition_boost_specs (MutableSequence[google.cloud.retail_v2alpha.types.SearchRequest.BoostSpec.ConditionBoostSpec]):
                Condition boost specifications. If a product
                matches multiple conditions in the
                specifictions, boost scores from these
                specifications are all applied and combined in a
                non-linear way. Maximum number of specifications
                is 20.
            skip_boost_spec_validation (bool):
                Whether to skip boostspec validation. If this field is set
                to true, invalid
                [BoostSpec.condition_boost_specs][google.cloud.retail.v2alpha.SearchRequest.BoostSpec.condition_boost_specs]
                will be ignored and valid
                [BoostSpec.condition_boost_specs][google.cloud.retail.v2alpha.SearchRequest.BoostSpec.condition_boost_specs]
                will still be applied.

                This field is a member of `oneof`_ ``_skip_boost_spec_validation``.
        """

        class ConditionBoostSpec(proto.Message):
            r"""Boost applies to products which match a condition.

            Attributes:
                condition (str):
                    An expression which specifies a boost condition. The syntax
                    and supported fields are the same as a filter expression.
                    See
                    [SearchRequest.filter][google.cloud.retail.v2alpha.SearchRequest.filter]
                    for detail syntax and limitations.

                    Examples:

                    -  To boost products with product ID "product_1" or
                       "product_2", and color "Red" or "Blue":

                       -  (id: ANY("product_1", "product_2")) AND
                          (colorFamilies: ANY("Red","Blue"))
                boost (float):
                    Strength of the condition boost, which should be in [-1, 1].
                    Negative boost means demotion. Default is 0.0.

                    Setting to 1.0 gives the item a big promotion. However, it
                    does not necessarily mean that the boosted item will be the
                    top result at all times, nor that other items will be
                    excluded. Results could still be shown even when none of
                    them matches the condition. And results that are
                    significantly more relevant to the search query can still
                    trump your heavily favored but irrelevant items.

                    Setting to -1.0 gives the item a big demotion. However,
                    results that are deeply relevant might still be shown. The
                    item will have an upstream battle to get a fairly high
                    ranking, but it is not blocked out completely.

                    Setting to 0.0 means no boost applied. The boosting
                    condition is ignored.
            """

            condition: str = proto.Field(
                proto.STRING,
                number=1,
            )
            boost: float = proto.Field(
                proto.FLOAT,
                number=2,
            )

        condition_boost_specs: MutableSequence[
            "SearchRequest.BoostSpec.ConditionBoostSpec"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="SearchRequest.BoostSpec.ConditionBoostSpec",
        )
        skip_boost_spec_validation: bool = proto.Field(
            proto.BOOL,
            number=2,
            optional=True,
        )

    class QueryExpansionSpec(proto.Message):
        r"""Specification to determine under which conditions query
        expansion should occur.

        Attributes:
            condition (google.cloud.retail_v2alpha.types.SearchRequest.QueryExpansionSpec.Condition):
                The condition under which query expansion should occur.
                Default to
                [Condition.DISABLED][google.cloud.retail.v2alpha.SearchRequest.QueryExpansionSpec.Condition.DISABLED].
            pin_unexpanded_results (bool):
                Whether to pin unexpanded results. If this
                field is set to true, unexpanded products are
                always at the top of the search results,
                followed by the expanded results.
        """

        class Condition(proto.Enum):
            r"""Enum describing under which condition query expansion should
            occur.

            Values:
                CONDITION_UNSPECIFIED (0):
                    Unspecified query expansion condition. In this case, server
                    behavior defaults to
                    [Condition.DISABLED][google.cloud.retail.v2alpha.SearchRequest.QueryExpansionSpec.Condition.DISABLED].
                DISABLED (1):
                    Disabled query expansion. Only the exact search query is
                    used, even if
                    [SearchResponse.total_size][google.cloud.retail.v2alpha.SearchResponse.total_size]
                    is zero.
                AUTO (3):
                    Automatic query expansion built by Google
                    Retail Search.
            """
            CONDITION_UNSPECIFIED = 0
            DISABLED = 1
            AUTO = 3

        condition: "SearchRequest.QueryExpansionSpec.Condition" = proto.Field(
            proto.ENUM,
            number=1,
            enum="SearchRequest.QueryExpansionSpec.Condition",
        )
        pin_unexpanded_results: bool = proto.Field(
            proto.BOOL,
            number=2,
        )

    class PersonalizationSpec(proto.Message):
        r"""The specification for personalization.

        Attributes:
            mode (google.cloud.retail_v2alpha.types.SearchRequest.PersonalizationSpec.Mode):
                Defaults to
                [Mode.AUTO][google.cloud.retail.v2alpha.SearchRequest.PersonalizationSpec.Mode.AUTO].
        """

        class Mode(proto.Enum):
            r"""The personalization mode of each search request.

            Values:
                MODE_UNSPECIFIED (0):
                    Default value. In this case, server behavior defaults to
                    [Mode.AUTO][google.cloud.retail.v2alpha.SearchRequest.PersonalizationSpec.Mode.AUTO].
                AUTO (1):
                    Let CRS decide whether to use personalization
                    based on quality of user event data.
                DISABLED (2):
                    Disable personalization.
            """
            MODE_UNSPECIFIED = 0
            AUTO = 1
            DISABLED = 2

        mode: "SearchRequest.PersonalizationSpec.Mode" = proto.Field(
            proto.ENUM,
            number=1,
            enum="SearchRequest.PersonalizationSpec.Mode",
        )

    class SpellCorrectionSpec(proto.Message):
        r"""The specification for query spell correction.

        Attributes:
            mode (google.cloud.retail_v2alpha.types.SearchRequest.SpellCorrectionSpec.Mode):
                The mode under which spell correction should take effect to
                replace the original search query. Default to
                [Mode.AUTO][google.cloud.retail.v2alpha.SearchRequest.SpellCorrectionSpec.Mode.AUTO].
        """

        class Mode(proto.Enum):
            r"""Enum describing under which mode spell correction should
            occur.

            Values:
                MODE_UNSPECIFIED (0):
                    Unspecified spell correction mode. In this case, server
                    behavior defaults to
                    [Mode.AUTO][google.cloud.retail.v2alpha.SearchRequest.SpellCorrectionSpec.Mode.AUTO].
                SUGGESTION_ONLY (1):
                    Google Retail Search will try to find a spell suggestion if
                    there is any and put in the
                    [SearchResponse.corrected_query][google.cloud.retail.v2alpha.SearchResponse.corrected_query].
                    The spell suggestion will not be used as the search query.
                AUTO (2):
                    Automatic spell correction built by Google
                    Retail Search. Search will be based on the
                    corrected query if found.
            """
            MODE_UNSPECIFIED = 0
            SUGGESTION_ONLY = 1
            AUTO = 2

        mode: "SearchRequest.SpellCorrectionSpec.Mode" = proto.Field(
            proto.ENUM,
            number=1,
            enum="SearchRequest.SpellCorrectionSpec.Mode",
        )

    placement: str = proto.Field(
        proto.STRING,
        number=1,
    )
    branch: str = proto.Field(
        proto.STRING,
        number=2,
    )
    query: str = proto.Field(
        proto.STRING,
        number=3,
    )
    visitor_id: str = proto.Field(
        proto.STRING,
        number=4,
    )
    user_info: common.UserInfo = proto.Field(
        proto.MESSAGE,
        number=5,
        message=common.UserInfo,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=7,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=8,
    )
    offset: int = proto.Field(
        proto.INT32,
        number=9,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=10,
    )
    canonical_filter: str = proto.Field(
        proto.STRING,
        number=28,
    )
    order_by: str = proto.Field(
        proto.STRING,
        number=11,
    )
    facet_specs: MutableSequence[FacetSpec] = proto.RepeatedField(
        proto.MESSAGE,
        number=12,
        message=FacetSpec,
    )
    dynamic_facet_spec: DynamicFacetSpec = proto.Field(
        proto.MESSAGE,
        number=21,
        message=DynamicFacetSpec,
    )
    boost_spec: BoostSpec = proto.Field(
        proto.MESSAGE,
        number=13,
        message=BoostSpec,
    )
    query_expansion_spec: QueryExpansionSpec = proto.Field(
        proto.MESSAGE,
        number=14,
        message=QueryExpansionSpec,
    )
    relevance_threshold: RelevanceThreshold = proto.Field(
        proto.ENUM,
        number=15,
        enum=RelevanceThreshold,
    )
    variant_rollup_keys: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=17,
    )
    page_categories: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=23,
    )
    search_mode: SearchMode = proto.Field(
        proto.ENUM,
        number=31,
        enum=SearchMode,
    )
    personalization_spec: PersonalizationSpec = proto.Field(
        proto.MESSAGE,
        number=32,
        message=PersonalizationSpec,
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=34,
    )
    spell_correction_spec: SpellCorrectionSpec = proto.Field(
        proto.MESSAGE,
        number=35,
        optional=True,
        message=SpellCorrectionSpec,
    )
    entity: str = proto.Field(
        proto.STRING,
        number=38,
    )


class SearchResponse(proto.Message):
    r"""Response message for
    [SearchService.Search][google.cloud.retail.v2alpha.SearchService.Search]
    method.

    Attributes:
        results (MutableSequence[google.cloud.retail_v2alpha.types.SearchResponse.SearchResult]):
            A list of matched items. The order represents
            the ranking.
        facets (MutableSequence[google.cloud.retail_v2alpha.types.SearchResponse.Facet]):
            Results of facets requested by user.
        total_size (int):
            The estimated total count of matched items irrespective of
            pagination. The count of
            [results][google.cloud.retail.v2alpha.SearchResponse.results]
            returned by pagination may be less than the
            [total_size][google.cloud.retail.v2alpha.SearchResponse.total_size]
            that matches.
        corrected_query (str):
            Contains the spell corrected query, if found. If the spell
            correction type is AUTOMATIC, then the search results are
            based on corrected_query. Otherwise the original query is
            used for search.
        attribution_token (str):
            A unique search token. This should be included in the
            [UserEvent][google.cloud.retail.v2alpha.UserEvent] logs
            resulting from this search, which enables accurate
            attribution of search model performance.
        next_page_token (str):
            A token that can be sent as
            [SearchRequest.page_token][google.cloud.retail.v2alpha.SearchRequest.page_token]
            to retrieve the next page. If this field is omitted, there
            are no subsequent pages.
        query_expansion_info (google.cloud.retail_v2alpha.types.SearchResponse.QueryExpansionInfo):
            Query expansion information for the returned
            results.
        redirect_uri (str):
            The URI of a customer-defined redirect page. If redirect
            action is triggered, no search is performed, and only
            [redirect_uri][google.cloud.retail.v2alpha.SearchResponse.redirect_uri]
            and
            [attribution_token][google.cloud.retail.v2alpha.SearchResponse.attribution_token]
            are set in the response.
        applied_controls (MutableSequence[str]):
            The fully qualified resource name of applied
            `controls <https://cloud.google.com/retail/docs/serving-control-rules>`__.
        invalid_condition_boost_specs (MutableSequence[google.cloud.retail_v2alpha.types.SearchRequest.BoostSpec.ConditionBoostSpec]):
            The invalid
            [SearchRequest.BoostSpec.condition_boost_specs][google.cloud.retail.v2alpha.SearchRequest.BoostSpec.condition_boost_specs]
            that are not applied during serving.
        experiment_info (MutableSequence[google.cloud.retail_v2alpha.types.ExperimentInfo]):
            Metadata related to A/B testing
            [Experiment][google.cloud.retail.v2alpha.Experiment]
            associated with this response. Only exists when an
            experiment is triggered.
    """

    class SearchResult(proto.Message):
        r"""Represents the search results.

        Attributes:
            id (str):
                [Product.id][google.cloud.retail.v2alpha.Product.id] of the
                searched [Product][google.cloud.retail.v2alpha.Product].
            product (google.cloud.retail_v2alpha.types.Product):
                The product data snippet in the search response. Only
                [Product.name][google.cloud.retail.v2alpha.Product.name] is
                guaranteed to be populated.

                [Product.variants][google.cloud.retail.v2alpha.Product.variants]
                contains the product variants that match the search query.
                If there are multiple product variants matching the query,
                top 5 most relevant product variants are returned and
                ordered by relevancy.

                If relevancy can be deternmined, use
                [matching_variant_fields][google.cloud.retail.v2alpha.SearchResponse.SearchResult.matching_variant_fields]
                to look up matched product variants fields. If relevancy
                cannot be determined, e.g. when searching "shoe" all
                products in a shoe product can be a match, 5 product
                variants are returned but order is meaningless.
            matching_variant_count (int):
                The count of matched
                [variant][google.cloud.retail.v2alpha.Product.Type.VARIANT]
                [Product][google.cloud.retail.v2alpha.Product]s.
            matching_variant_fields (MutableMapping[str, google.protobuf.field_mask_pb2.FieldMask]):
                If a
                [variant][google.cloud.retail.v2alpha.Product.Type.VARIANT]
                [Product][google.cloud.retail.v2alpha.Product] matches the
                search query, this map indicates which
                [Product][google.cloud.retail.v2alpha.Product] fields are
                matched. The key is the
                [Product.name][google.cloud.retail.v2alpha.Product.name],
                the value is a field mask of the matched
                [Product][google.cloud.retail.v2alpha.Product] fields. If
                matched attributes cannot be determined, this map will be
                empty.

                For example, a key "sku1" with field mask
                "products.color_info" indicates there is a match between
                "sku1" [ColorInfo][google.cloud.retail.v2alpha.ColorInfo]
                and the query.
            variant_rollup_values (MutableMapping[str, google.protobuf.struct_pb2.Value]):
                The rollup matching
                [variant][google.cloud.retail.v2alpha.Product.Type.VARIANT]
                [Product][google.cloud.retail.v2alpha.Product] attributes.
                The key is one of the
                [SearchRequest.variant_rollup_keys][google.cloud.retail.v2alpha.SearchRequest.variant_rollup_keys].
                The values are the merged and de-duplicated
                [Product][google.cloud.retail.v2alpha.Product] attributes.
                Notice that the rollup values are respect filter. For
                example, when filtering by "colorFamilies:ANY("red")" and
                rollup "colorFamilies", only "red" is returned.

                For textual and numerical attributes, the rollup values is a
                list of string or double values with type
                [google.protobuf.ListValue][google.protobuf.ListValue]. For
                example, if there are two variants with colors "red" and
                "blue", the rollup values are

                ::

                    { key: "colorFamilies"
                      value {
                        list_value {
                          values { string_value: "red" }
                          values { string_value: "blue" }
                         }
                      }
                    }

                For
                [FulfillmentInfo][google.cloud.retail.v2alpha.FulfillmentInfo],
                the rollup values is a double value with type
                [google.protobuf.Value][google.protobuf.Value]. For example,
                ``{key: "pickupInStore.store1" value { number_value: 10 }}``
                means a there are 10 variants in this product are available
                in the store "store1".
            personal_labels (MutableSequence[str]):
                Specifies previous events related to this product for this
                user based on
                [UserEvent][google.cloud.retail.v2alpha.UserEvent] with same
                [SearchRequest.visitor_id][google.cloud.retail.v2alpha.SearchRequest.visitor_id]
                or
                [UserInfo.user_id][google.cloud.retail.v2alpha.UserInfo.user_id].

                This is set only when
                [SearchRequest.PersonalizationSpec.mode][google.cloud.retail.v2alpha.SearchRequest.PersonalizationSpec.mode]
                is
                [SearchRequest.PersonalizationSpec.Mode.AUTO][google.cloud.retail.v2alpha.SearchRequest.PersonalizationSpec.Mode.AUTO].

                Possible values:

                -  ``purchased``: Indicates that this product has been
                   purchased before.
        """

        id: str = proto.Field(
            proto.STRING,
            number=1,
        )
        product: gcr_product.Product = proto.Field(
            proto.MESSAGE,
            number=2,
            message=gcr_product.Product,
        )
        matching_variant_count: int = proto.Field(
            proto.INT32,
            number=3,
        )
        matching_variant_fields: MutableMapping[
            str, field_mask_pb2.FieldMask
        ] = proto.MapField(
            proto.STRING,
            proto.MESSAGE,
            number=4,
            message=field_mask_pb2.FieldMask,
        )
        variant_rollup_values: MutableMapping[str, struct_pb2.Value] = proto.MapField(
            proto.STRING,
            proto.MESSAGE,
            number=5,
            message=struct_pb2.Value,
        )
        personal_labels: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=7,
        )

    class Facet(proto.Message):
        r"""A facet result.

        Attributes:
            key (str):
                The key for this facet. E.g., "colorFamilies"
                or "price" or "attributes.attr1".
            values (MutableSequence[google.cloud.retail_v2alpha.types.SearchResponse.Facet.FacetValue]):
                The facet values for this field.
            dynamic_facet (bool):
                Whether the facet is dynamically generated.
        """

        class FacetValue(proto.Message):
            r"""A facet value which contains value names and their count.

            This message has `oneof`_ fields (mutually exclusive fields).
            For each oneof, at most one member field can be set at the same time.
            Setting any member of the oneof automatically clears all other
            members.

            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                value (str):
                    Text value of a facet, such as "Black" for
                    facet "colorFamilies".

                    This field is a member of `oneof`_ ``facet_value``.
                interval (google.cloud.retail_v2alpha.types.Interval):
                    Interval value for a facet, such as [10, 20) for facet
                    "price".

                    This field is a member of `oneof`_ ``facet_value``.
                count (int):
                    Number of items that have this facet value.
                min_value (float):
                    The minimum value in the
                    [FacetValue.interval][google.cloud.retail.v2alpha.SearchResponse.Facet.FacetValue.interval].
                    Only supported on numerical facets and returned if
                    [SearchRequest.FacetSpec.FacetKey.return_min_max][google.cloud.retail.v2alpha.SearchRequest.FacetSpec.FacetKey.return_min_max]
                    is true.
                max_value (float):
                    The maximum value in the
                    [FacetValue.interval][google.cloud.retail.v2alpha.SearchResponse.Facet.FacetValue.interval].
                    Only supported on numerical facets and returned if
                    [SearchRequest.FacetSpec.FacetKey.return_min_max][google.cloud.retail.v2alpha.SearchRequest.FacetSpec.FacetKey.return_min_max]
                    is true.
            """

            value: str = proto.Field(
                proto.STRING,
                number=1,
                oneof="facet_value",
            )
            interval: common.Interval = proto.Field(
                proto.MESSAGE,
                number=2,
                oneof="facet_value",
                message=common.Interval,
            )
            count: int = proto.Field(
                proto.INT64,
                number=3,
            )
            min_value: float = proto.Field(
                proto.DOUBLE,
                number=5,
            )
            max_value: float = proto.Field(
                proto.DOUBLE,
                number=6,
            )

        key: str = proto.Field(
            proto.STRING,
            number=1,
        )
        values: MutableSequence[
            "SearchResponse.Facet.FacetValue"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="SearchResponse.Facet.FacetValue",
        )
        dynamic_facet: bool = proto.Field(
            proto.BOOL,
            number=3,
        )

    class QueryExpansionInfo(proto.Message):
        r"""Information describing query expansion including whether
        expansion has occurred.

        Attributes:
            expanded_query (bool):
                Bool describing whether query expansion has
                occurred.
            pinned_result_count (int):
                Number of pinned results. This field will only be set when
                expansion happens and
                [SearchRequest.QueryExpansionSpec.pin_unexpanded_results][google.cloud.retail.v2alpha.SearchRequest.QueryExpansionSpec.pin_unexpanded_results]
                is set to true.
        """

        expanded_query: bool = proto.Field(
            proto.BOOL,
            number=1,
        )
        pinned_result_count: int = proto.Field(
            proto.INT64,
            number=2,
        )

    @property
    def raw_page(self):
        return self

    results: MutableSequence[SearchResult] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=SearchResult,
    )
    facets: MutableSequence[Facet] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=Facet,
    )
    total_size: int = proto.Field(
        proto.INT32,
        number=3,
    )
    corrected_query: str = proto.Field(
        proto.STRING,
        number=4,
    )
    attribution_token: str = proto.Field(
        proto.STRING,
        number=5,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=6,
    )
    query_expansion_info: QueryExpansionInfo = proto.Field(
        proto.MESSAGE,
        number=7,
        message=QueryExpansionInfo,
    )
    redirect_uri: str = proto.Field(
        proto.STRING,
        number=10,
    )
    applied_controls: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=12,
    )
    invalid_condition_boost_specs: MutableSequence[
        "SearchRequest.BoostSpec.ConditionBoostSpec"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=14,
        message="SearchRequest.BoostSpec.ConditionBoostSpec",
    )
    experiment_info: MutableSequence["ExperimentInfo"] = proto.RepeatedField(
        proto.MESSAGE,
        number=17,
        message="ExperimentInfo",
    )


class ExperimentInfo(proto.Message):
    r"""Metadata for active A/B testing [Experiments][].

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        serving_config_experiment (google.cloud.retail_v2alpha.types.ExperimentInfo.ServingConfigExperiment):
            A/B test between existing Cloud Retail Search
            [ServingConfig][google.cloud.retail.v2alpha.ServingConfig]s.

            This field is a member of `oneof`_ ``experiment_metadata``.
        experiment (str):
            The fully qualified resource name of the experiment that
            provides the serving config under test, should an active
            experiment exist. For example:
            ``projects/*/locations/global/catalogs/default_catalog/experiments/experiment_id``
    """

    class ServingConfigExperiment(proto.Message):
        r"""Metadata for active serving config A/B tests.

        Attributes:
            original_serving_config (str):
                The fully qualified resource name of the original
                [SearchRequest.placement][google.cloud.retail.v2alpha.SearchRequest.placement]
                in the search request prior to reassignment by experiment
                API. For example:
                ``projects/*/locations/*/catalogs/*/servingConfigs/*``.
            experiment_serving_config (str):
                The fully qualified resource name of the serving config
                [VariantArm.serving_config_id][] responsible for generating
                the search response. For example:
                ``projects/*/locations/*/catalogs/*/servingConfigs/*``.
        """

        original_serving_config: str = proto.Field(
            proto.STRING,
            number=1,
        )
        experiment_serving_config: str = proto.Field(
            proto.STRING,
            number=2,
        )

    serving_config_experiment: ServingConfigExperiment = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="experiment_metadata",
        message=ServingConfigExperiment,
    )
    experiment: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import timestamp_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.retail_v2alpha.types import common

__protobuf__ = proto.module(
    package="google.cloud.retail.v2alpha",
    manifest={
        "Model",
    },
)


class Model(proto.Message):
    r"""Metadata that describes the training and serving parameters of a
    [Model][google.cloud.retail.v2alpha.Model]. A
    [Model][google.cloud.retail.v2alpha.Model] can be associated with a
    [ServingConfig][google.cloud.retail.v2alpha.ServingConfig] and then
    queried through the Predict API.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        page_optimization_config (google.cloud.retail_v2alpha.types.Model.PageOptimizationConfig):
            Optional. The page optimization config.

            This field is a member of `oneof`_ ``training_config``.
        name (str):
            Required. The fully qualified resource name of the model.

            Format:
            ``projects/{project_number}/locations/{location_id}/catalogs/{catalog_id}/models/{model_id}``
            catalog_id has char limit of 50. recommendation_model_id has
            char limit of 40.
        display_name (str):
            Required. The display name of the model.

            Should be human readable, used to display
            Recommendation Models in the Retail Cloud
            Console Dashboard. UTF-8 encoded string with
            limit of 1024 characters.
        training_state (google.cloud.retail_v2alpha.types.Model.TrainingState):
            Optional. The training state that the model is in (e.g.
            ``TRAINING`` or ``PAUSED``).

            Since part of the cost of running the service is frequency
            of training - this can be used to determine when to train
            model in order to control cost. If not specified: the
            default value for ``CreateModel`` method is ``TRAINING``.
            The default value for ``UpdateModel`` method is to keep the
            state the same as before.
        serving_state (google.cloud.retail_v2alpha.types.Model.ServingState):
            Output only. The serving state of the model: ``ACTIVE``,
            ``NOT_ACTIVE``.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Timestamp the Recommendation
            Model was created at.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Timestamp the Recommendation
            Model was last updated. E.g. if a Recommendation
            Model was paused - this would be the time the
            pause was initiated.
        type_ (str):
            Required. The type of model e.g. ``home-page``.

            Currently supported values: ``recommended-for-you``,
            ``others-you-may-like``, ``frequently-bought-together``,
            ``page-optimization``, ``similar-items``, ``buy-it-again``,
            ``on-sale-items``, and ``recently-viewed``\ (readonly
            value).

            This field together with
            [optimization_objective][google.cloud.retail.v2alpha.Model.optimization_objective]
            describe model metadata to use to control model training and
            serving. See https://cloud.google.com/retail/docs/models for
            more details on what the model metadata control and which
            combination of parameters are valid. For invalid
            combinations of parameters (e.g. type =
            ``frequently-bought-together`` and optimization_objective =
            ``ctr``), you receive an error 400 if you try to
            create/update a recommendation with this set of knobs.
        optimization_objective (str):
            Optional. The optimization objective e.g. ``cvr``.

            Currently supported values: ``ctr``, ``cvr``,
            ``revenue-per-order``.

            If not specified, we choose default based on model type.
            Default depends on type of recommendation:

            ``recommended-for-you`` => ``ctr``

            ``others-you-may-like`` => ``ctr``

            ``frequently-bought-together`` => ``revenue_per_order``

            This field together with
            [optimization_objective][google.cloud.retail.v2alpha.Model.type]
            describe model metadata to use to control model training and
            serving. See https://cloud.google.com/retail/docs/models for
            more details on what the model metadata control and which
            combination of parameters are valid. For invalid
            combinations of parameters (e.g. type =
            ``frequently-bought-together`` and optimization_objective =
            ``ctr``), you receive an error 400 if you try to
            create/update a recommendation with this set of knobs.
        periodic_tuning_state (google.cloud.retail_v2alpha.types.Model.PeriodicTuningState):
            Optional. The state of periodic tuning.

            The period we use is 3 months - to do a one-off tune earlier
            use the ``TuneModel`` method. Default value is
            ``PERIODIC_TUNING_ENABLED``.
        last_tune_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The timestamp when the latest
            successful tune finished.
        tuning_operation (str):
            Output only. The tune operation associated
            with the model.
            Can be used to determine if there is an ongoing
            tune for this recommendation. Empty field
            implies no tune is goig on.
        data_state (google.cloud.retail_v2alpha.types.Model.DataState):
            Output only. The state of data requirements for this model:
            ``DATA_OK`` and ``DATA_ERROR``.

            Recommendation model cannot be trained if the data is in
            ``DATA_ERROR`` state. Recommendation model can have
            ``DATA_ERROR`` state even if serving state is ``ACTIVE``:
            models were trained successfully before, but cannot be
            refreshed because model no longer has sufficient data for
            training.
        filtering_option (google.cloud.retail_v2alpha.types.RecommendationsFilteringOption):
            Optional. If ``RECOMMENDATIONS_FILTERING_ENABLED``,
            recommendation filtering by attributes is enabled for the
            model.
        serving_config_lists (MutableSequence[google.cloud.retail_v2alpha.types.Model.ServingConfigList]):
            Output only. The list of valid serving
            configs associated with the
            PageOptimizationConfig.
    """

    class ServingState(proto.Enum):
        r"""The serving state of the model.

        Values:
            SERVING_STATE_UNSPECIFIED (0):
                Unspecified serving state.
            INACTIVE (1):
                The model is not serving.
            ACTIVE (2):
                The model is serving and can be queried.
            TUNED (3):
                The model is trained on tuned hyperparameters
                and can be queried.
        """
        SERVING_STATE_UNSPECIFIED = 0
        INACTIVE = 1
        ACTIVE = 2
        TUNED = 3

    class TrainingState(proto.Enum):
        r"""The training state of the model.

        Values:
            TRAINING_STATE_UNSPECIFIED (0):
                Unspecified training state.
            PAUSED (1):
                The model training is paused.
            TRAINING (2):
                The model is training.
        """
        TRAINING_STATE_UNSPECIFIED = 0
        PAUSED = 1
        TRAINING = 2

    class PeriodicTuningState(proto.Enum):
        r"""Describes whether periodic tuning is enabled for this model or not.
        Periodic tuning is scheduled at most every three months. You can
        start a tuning process manually by using the ``TuneModel`` method,
        which starts a tuning process immediately and resets the quarterly
        schedule. Enabling or disabling periodic tuning does not affect any
        current tuning processes.

        Values:
            PERIODIC_TUNING_STATE_UNSPECIFIED (0):
                Unspecified default value, should never be
                explicitly set.
            PERIODIC_TUNING_DISABLED (1):
                The model has periodic tuning disabled. Tuning can be
                reenabled by calling the ``EnableModelPeriodicTuning``
                method or by calling the ``TuneModel`` method.
            ALL_TUNING_DISABLED (3):
                The model cannot be tuned with periodic tuning OR the
                ``TuneModel`` method. Hide the options in customer UI and
                reject any requests through the backend self serve API.
            PERIODIC_TUNING_ENABLED (2):
                The model has periodic tuning enabled. Tuning can be
                disabled by calling the ``DisableModelPeriodicTuning``
                method.
        """
        PERIODIC_TUNING_STATE_UNSPECIFIED = 0
        PERIODIC_TUNING_DISABLED = 1
        ALL_TUNING_DISABLED = 3
        PERIODIC_TUNING_ENABLED = 2

    class DataState(proto.Enum):
        r"""Describes whether this model have sufficient training data
        to be continuously trained.

        Values:
            DATA_STATE_UNSPECIFIED (0):
                Unspecified default value, should never be
                explicitly set.
            DATA_OK (1):
                The model has sufficient training data.
            DATA_ERROR (2):
                The model does not have sufficient training
                data. Error messages can be queried via
                Stackdriver.
        """
        DATA_STATE_UNSPECIFIED = 0
        DATA_OK = 1
        DATA_ERROR = 2

    class PageOptimizationConfig(proto.Message):
        r"""The PageOptimizationConfig for model training.

        This determines how many panels to optimize for, and which serving
        configs to consider for each panel. The purpose of this model is to
        optimize which
        [ServingConfig][google.cloud.retail.v2alpha.ServingConfig] to show
        on which panels in way that optimizes the visitors shopping journey.

        Attributes:
            page_optimization_event_type (str):
                Required. The type of
                [UserEvent][google.cloud.retail.v2alpha.UserEvent] this page
                optimization is shown for.

                Each page has an associated event type - this will be the
                corresponding event type for the page that the page
                optimization model is used on.

                Supported types:

                -  ``add-to-cart``: Products being added to cart.
                -  ``detail-page-view``: Products detail page viewed.
                -  ``home-page-view``: Homepage viewed
                -  ``category-page-view``: Homepage viewed
                -  ``shopping-cart-page-view``: User viewing a shopping
                   cart.

                ``home-page-view`` only allows models with type
                ``recommended-for-you``. All other
                page_optimization_event_type allow all
                [Model.types][google.cloud.retail.v2alpha.Model.type].
            panels (MutableSequence[google.cloud.retail_v2alpha.types.Model.PageOptimizationConfig.Panel]):
                Required. A list of panel configurations.

                Limit = 5.
            restriction (google.cloud.retail_v2alpha.types.Model.PageOptimizationConfig.Restriction):
                Optional. How to restrict results across panels e.g. can the
                same
                [ServingConfig][google.cloud.retail.v2alpha.ServingConfig]
                be shown on multiple panels at once.

                If unspecified, default to ``UNIQUE_MODEL_RESTRICTION``.
        """

        class Restriction(proto.Enum):
            r"""Restrictions of expected returned results.

            Values:
                RESTRICTION_UNSPECIFIED (0):
                    Unspecified value for restriction.
                NO_RESTRICTION (1):
                    Allow any
                    [ServingConfig][google.cloud.retail.v2alpha.ServingConfig]
                    to be show on any number of panels.

                    Example:

                    ``Panel1 candidates``: pdp_ctr, pdp_cvr,
                    home_page_ctr_no_diversity

                    ``Panel2 candidates``: home_page_ctr_no_diversity,
                    home_page_ctr_diversity, pdp_cvr_no_diversity

                    ``Restriction`` = NO_RESTRICTION

                    ``Valid combinations``:

                    -   (pdp_ctr, home_page_ctr_no_diversity)
                    -  (pdp_ctr, home_page_ctr_diversity)
                    -  (pdp_ctr, pdp_cvr_no_diversity)
                    -  (pdp_cvr, home_page_ctr_no_diversity)
                    -  (pdp_cvr, home_page_ctr_diversity)
                    -  (pdp_cvr, pdp_cvr_no_diversity)
                    -  (home_page_ctr_no_diversity, home_page_ctr_no_diversity)
                    -  (home_page_ctr_no_diversity, home_page_ctr_diversity)
                    -  (home_page_ctr_no_diversity, pdp_cvr_no_diversity)

                    ``Invalid combinations``: []
                UNIQUE_SERVING_CONFIG_RESTRICTION (2):
                    Do not allow the same
                    [ServingConfig.name][google.cloud.retail.v2alpha.ServingConfig.name]
                    to be shown on multiple panels.

                    Example:

                    ``Panel1 candidates``: pdp_ctr, pdp_cvr,
                    home_page_ctr_no_diversity

                    ``Panel2 candidates``: home_page_ctr_no_diversity,
                    home_page_ctr_diversity_low, pdp_cvr_no_diversity

                    ``Restriction`` = ``UNIQUE_SERVING_CONFIG_RESTRICTION``

                    ``Valid combinations``:

                    -   (pdp_ctr, home_page_ctr_no_diversity)
                    -  (pdp_ctr, home_page_ctr_diversity_low)
                    -  (pdp_ctr, pdp_cvr_no_diversity)
                    -  (pdp_ctr, pdp_cvr_no_diversity)
                    -  (pdp_cvr, home_page_ctr_no_diversity)
                    -  (pdp_cvr, home_page_ctr_diversity_low)
                    -  (pdp_cvr, pdp_cvr_no_diversity)
                    -  (home_page_ctr_no_diversity, home_page_ctr_diversity_low)
                    -  (home_page_ctr_no_diversity, pdp_cvr_no_diversity)

                    ``Invalid combinations``:

                    -   (home_page_ctr_no_diversity, home_page_ctr_no_diversity)
                UNIQUE_MODEL_RESTRICTION (3):
                    Do not allow multiple
                    [ServingConfigs][google.cloud.retail.v2alpha.ServingConfig]
                    with same
                    [Model.name][google.cloud.retail.v2alpha.Model.name] to be
                    show on on different panels.

                    Example:

                    ``Panel1 candidates``: pdp_ctr, pdp_cvr,
                    home_page_ctr_no_diversity

                    ``Panel2 candidates``: home_page_ctr_no_diversity,
                    home_page_ctr_diversity_low, pdp_cvr_no_diversity

                    ``Restriction`` = ``UNIQUE_MODEL_RESTRICTION``

                    ``Valid combinations``:

                    -   (pdp_ctr, home_page_ctr_no_diversity)
                    -  (pdp_ctr, home_page_ctr_diversity)
                    -  (pdp_ctr, pdp_cvr_no_diversity)
                    -  (pdp_ctr, pdp_cvr_no_diversity)
                    -  (pdp_cvr, home_page_ctr_no_diversity)
                    -  (pdp_cvr, home_page_ctr_diversity_low)
                    -  (home_page_ctr_no_diversity, pdp_cvr_no_diversity)

                    ``Invalid combinations``:

                    -   (home_page_ctr_no_diversity, home_page_ctr_no_diversity)
                    -  (pdp_cvr, pdp_cvr_no_diversity)
                UNIQUE_MODEL_TYPE_RESTRICTION (4):
                    Do not allow multiple
                    [ServingConfigs][google.cloud.retail.v2alpha.ServingConfig]
                    with same
                    [Model.type][google.cloud.retail.v2alpha.Model.type] to be
                    shown on different panels.

                    Example:

                    ``Panel1 candidates``: pdp_ctr, pdp_cvr,
                    home_page_ctr_no_diversity

                    ``Panel2 candidates``: home_page_ctr_no_diversity,
                    home_page_ctr_diversity_low, pdp_cvr_no_diversity

                    ``Restriction`` = ``UNIQUE_MODEL_RESTRICTION``

                    ``Valid combinations``:

                    -   (pdp_ctr, home_page_ctr_no_diversity)
                    -  (pdp_ctr, home_page_ctr_diversity)
                    -  (pdp_cvr, home_page_ctr_no_diversity)
                    -  (pdp_cvr, home_page_ctr_diversity_low)
                    -  (home_page_ctr_no_diversity, pdp_cvr_no_diversity)

                    ``Invalid combinations``:

                    -   (pdp_ctr, pdp_cvr_no_diversity)
                    -  (pdp_ctr, pdp_cvr_no_diversity)
                    -  (pdp_cvr, pdp_cvr_no_diversity)
                    -  (home_page_ctr_no_diversity, home_page_ctr_no_diversity)
                    -  (home_page_ctr_no_diversity, home_page_ctr_diversity)
            """
            RESTRICTION_UNSPECIFIED = 0
            NO_RESTRICTION = 1
            UNIQUE_SERVING_CONFIG_RESTRICTION = 2
            UNIQUE_MODEL_RESTRICTION = 3
            UNIQUE_MODEL_TYPE_RESTRICTION = 4

        class Candidate(proto.Message):
            r"""A candidate to consider for a given panel. Currently only
            [ServingConfig][google.cloud.retail.v2alpha.ServingConfig] are valid
            candidates.


            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                serving_config_id (str):
                    This has to be a valid
                    [ServingConfig][google.cloud.retail.v2alpha.ServingConfig]
                    identifier. For example, for a ServingConfig with full name:
                    ``projects/*/locations/global/catalogs/default_catalog/servingConfigs/my_candidate_config``,
                    this would be ``my_candidate_config``.

                    This field is a member of `oneof`_ ``candidate``.
            """

            serving_config_id: str = proto.Field(
                proto.STRING,
                number=1,
                oneof="candidate",
            )

        class Panel(proto.Message):
            r"""An individual panel with a list of
            [ServingConfigs][google.cloud.retail.v2alpha.ServingConfig] to
            consider for it.

            Attributes:
                display_name (str):
                    Optional. The name to display for the panel.
                candidates (MutableSequence[google.cloud.retail_v2alpha.types.Model.PageOptimizationConfig.Candidate]):
                    Required. The candidates to consider on the
                    panel.
                default_candidate (google.cloud.retail_v2alpha.types.Model.PageOptimizationConfig.Candidate):
                    Required. The default candidate. If the model
                    fails at serving time, we fall back to the
                    default.
            """

            display_name: str = proto.Field(
                proto.STRING,
                number=1,
            )
            candidates: MutableSequence[
                "Model.PageOptimizationConfig.Candidate"
            ] = proto.RepeatedField(
                proto.MESSAGE,
                number=2,
                message="Model.PageOptimizationConfig.Candidate",
            )
            default_candidate: "Model.PageOptimizationConfig.Candidate" = proto.Field(
                proto.MESSAGE,
                number=3,
                message="Model.PageOptimizationConfig.Candidate",
            )

        page_optimization_event_type: str = proto.Field(
            proto.STRING,
            number=1,
        )
        panels: MutableSequence[
            "Model.PageOptimizationConfig.Panel"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="Model.PageOptimizationConfig.Panel",
        )
        restriction: "Model.PageOptimizationConfig.Restriction" = proto.Field(
            proto.ENUM,
            number=3,
            enum="Model.PageOptimizationConfig.Restriction",
        )

    class ServingConfigList(proto.Message):
        r"""Represents an ordered combination of valid serving configs, which
        can be used for ``PAGE_OPTIMIZATION`` recommendations.

        Attributes:
            serving_config_ids (MutableSequence[str]):
                Optional. A set of valid serving configs that may be used
                for ``PAGE_OPTIMIZATION``.
        """

        serving_config_ids: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=1,
        )

    page_optimization_config: PageOptimizationConfig = proto.Field(
        proto.MESSAGE,
        number=17,
        oneof="training_config",
        message=PageOptimizationConfig,
    )
    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    training_state: TrainingState = proto.Field(
        proto.ENUM,
        number=3,
        enum=TrainingState,
    )
    serving_state: ServingState = proto.Field(
        proto.ENUM,
        number=4,
        enum=ServingState,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=6,
        message=timestamp_pb2.Timestamp,
    )
    type_: str = proto.Field(
        proto.STRING,
        number=7,
    )
    optimization_objective: str = proto.Field(
        proto.STRING,
        number=8,
    )
    periodic_tuning_state: PeriodicTuningState = proto.Field(
        proto.ENUM,
        number=11,
        enum=PeriodicTuningState,
    )
    last_tune_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=12,
        message=timestamp_pb2.Timestamp,
    )
    tuning_operation: str = proto.Field(
        proto.STRING,
        number=15,
    )
    data_state: DataState = proto.Field(
        proto.ENUM,
        number=16,
        enum=DataState,
    )
    filtering_option: common.RecommendationsFilteringOption = proto.Field(
        proto.ENUM,
        number=18,
        enum=common.RecommendationsFilteringOption,
    )
    serving_config_lists: MutableSequence[ServingConfigList] = proto.RepeatedField(
        proto.MESSAGE,
        number=19,
        message=ServingConfigList,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
